###########################################################################
#  Vintel - Visual Intel Chat Analyzer                                    #
#  Copyright (C) 2014-15 Sebastian Meyer (sparrow.242.de+eve@gmail.com )  #
#                                                                         #
#  This program is free software: you can redistribute it and/or modify   #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation, either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful,        #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#  GNU General Public License for more details.                           #
#                                                                         #
#                                                                         #
#  You should have received a copy of the GNU General Public License      #
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
###########################################################################

import datetime
import urllib
import urllib2

from PyQt4.QtGui import QImage, QPixmap

from bs4 import BeautifulSoup


ERROR = -1
NOT_EXISTS = 0
EXISTS = 1


def charname_to_id(name):
    """ Uses the EVE API to convert a charname to his ID
    """
    try:
        url = "https://api.eveonline.com//eve/CharacterID.xml.aspx"
        data = urllib.urlencode({"names" : name})
        request = urllib2.urlopen(url=url, data=data)
        content = request.read()
        soup = BeautifulSoup(content)
        rowset = soup.select("rowset")[0]
        for row in rowset.select("row"):
            if row["name"] == name:
                return int(row["characterid"])
    except Exception as e:
        print("Exception turning charname to id via API: {0}".format(str(e)))
        # fallback! if there is a problem with the API, we use evegate
        try:
            base_url = "https://gate.eveonline.com/Profile/"
            qcharname = urllib2.quote(name)
            url = base_url + qcharname
            content = urllib2.urlopen(url).read()
            soup = BeautifulSoup(content)
            img= soup.select("#imgActiveCharacter")
            image_url = soup.select("#imgActiveCharacter")[0]["src"]
            return image_url[image_url.rfind("/")+1:image_url.rfind("_")]
        except Exception as e:
            print "Exception during fallback char to id via evegate:", str(e)
            

def get_avatar_for_player(charname):
    """ Downlaoding th eavatar for a player/character
        charname = name of the character
        returns None if something gone wrong
    """ 
    avatar = None
    try:
        char_id = charname_to_id(charname)
        if char_id:
            image_url = "http://image.eveonline.com/Character/{id}_{size}.jpg"
            avatar = urllib2.urlopen(image_url.format(id=char_id, 
                                                      size=32)).read()
    except Exception as e:
        print "Exception during get_avatar_for_player:", str(e)
        avatar = None
    return avatar


def check_playername(charname):
    """ Checking on evegate for an exiting playername 
        returns 1 if exists, 0 if not and -1 if an error occured
    """
    base_url = "https://gate.eveonline.com/Profile/"
    qcharname = urllib2.quote(charname)
    url = base_url + qcharname
    retvalue = -1
    try:
        urllib2.urlopen(url)
        retvalue = 1
    except urllib2.HTTPError as e:
        if ("404") in str(e):
            retvalue = 0
    except Exception as e:
        print "Exception on check playername: ", str(e)
    return retvalue


def current_eve_time():
    """ returns the current eve-time as a datetime.datetime"""
    return datetime.datetime.utcnow()


def eve_epoch():
    """ returns the seconds since epoch in eve timezone"""
    return time.mktime(datetime.datetime.utcnow().timetuple())


def seconds_till_downtime():
    """ return the seconds till the next downtime"""
    now = current_eve_time()
    target = now
    if now.hour > 11:
        target = target + datetime.timedelta(1)
    target = datetime.datetime(target.year, target.month, target.day,
                               11, 0, 0, 0)
    delta = target - now
    return delta.seconds
    

SHIPNAMES = [u'ABADDON', u'ABSOLUTION', u'AEON', u'ALGOS', u'ANATHEMA',
             u'ANSHAR', u'APOCALYPSE', u'APOCALYPSE IMPERIAL ISSUE',
             u'APOTHEOSIS', u'ARAZU', u'ARBITRATOR', u'ARCHON', u'ARES',
             u'ARK', u'ARMAGEDDON', u'ASHIMMU', u'ASTARTE', u'ASTERO',
             u'ATRON', u'AUGOROR', u'AUGOROR NAVY ISSUE', u'AVATAR',
             u'BADGER', u'BANTAM', u'BASILISK', u'BELLICOSE', u'BESTOWER',
             u'BHAALGORN', u'BLACKBIRD', u'BREACHER', u'BROADSWORD', u'BRUTIX',
             u'BURST', u'BUSTARD', u'BUZZARD', u'CALDARI NAVY HOOKBILL',
             u'CARACAL', u'CATALYST', u'CELESTIS', u'CERBERUS', u'CHEETAH',
             u'CHIMERA', u'CLAW', u'CLAYMORE', u'COERCER', u'CONDOR',
             u'CONFESSOR', u'CORAX', u'CORMORANT', u'COVETOR', u'CRANE',
             u'CROW', u'CRUCIFIER', u'CRUOR', u'CRUSADER', u'CURSE',
             u'CYCLONE', u'CYNABAL', u'DAMNATION', u'DAREDEVIL', u'DEIMOS',
             u'DEVOTER', u'DOMINIX', u'DRAGOON', u'DRAKE', u'DRAMIEL',
             u'EAGLE', u'ENYO', u'EOS', u'EREBUS', u'ERIS', u'EXECUTIONER',
             u'EXEQUROR', u'EXEQUROR NAVY ISSUE', u'FALCON', u'FEROX',
             u'FLYCATCHER', u'FEDERATION NAVY COMET', u'GILA', u'GNOSIS',
             u'GOLD MAGNATE', u'GOLEM', u"GORU'S SHUTTLE", u'GRIFFIN',
             u'GUARDIAN', u'GUARDIAN-VEXOR', u'GURISTAS SHUTTLE',
             u'HARBINGER', u'HARPY', u'HAWK', u'HEL', u'HELIOS', u'HERETIC',
             u'HERON', u'HOARDER', u'HOUND', u'HUGINN', u'HULK', u'HURRICANE',
             u'HYENA', u'HYPERION', u'IBIS', u'IMICUS', u'IMPAIROR', u'IMPEL',
             u'IMPERIAL NAVY SLICER', u'INCURSUS', u'INQUISITOR', u'ISHKUR',
             u'ISHTAR', u'ITERON', u'JAGUAR', u'KERES', u'KESTREL', u'KITSUNE',
             u'KRONOS', u'LACHESIS', u'LEGION', u'LEVIATHAN', u'LOKI',
             u'MACHARIEL', u'MACKINAW', u'MAELSTROM', u'MAGNATE',
             u'MALEDICTION', u'MALLER', u'MAMMOTH', u'MANTICORE', u'MASTODON',
             u'MAULUS', u'MEGATHRON', u'MEGATHRON FEDERATE ISSUE',
             u'MEGATHRON NAVY ISSUE', u'MERLIN', u'MOA', u'MOROS', u'MUNINN',
             u'MYRMIDON', u'NAGA', u'NAGLFAR', u'NAVITAS', u'NEMESIS',
             u'NIDHOGGUR', u'NIGHTHAWK', u'NIGHTMARE', u'NOMAD', u'NYX',
             u'OCCATOR', u'OMEN', u'OMEN NAVY ISSUE', u'ONEIROS', u'ONYX',
             u'ORACLE', u'ORCA', u'OSPREY', u'OSPREY NAVY ISSUE', u'PALADIN',
             u'PANTHER', u'PHANTASM', u'PHOBOS', u'PHOENIX', u'PILGRIM',
             u'PRORATOR', u'PROBE', u'PROCURER', u'PROPHECY', u'PROTEUS',
             u'PROWLER', u'PUNISHER', u'PURIFIER', u'RAGNAROK', u'RAPIER',
             u'RAPTOR', u'RATTLESNAKE', u'RAVEN', u'RAVEN NAVY ISSUE',
             u'RAVEN STATE ISSUE', u'REAPER', u'REDEEMER',
             u'REPUBLIC FLEET FIRETAIL', u'RETRIBUTION', u'RETRIEVER',
             u'REVELATION', u'RHEA', u'RIFTER', u'ROKH', u'ROOK', u'RORQUAL',
             u'RUPTURE', u'SABRE', u'SACRILEGE', u'SCIMITAR', u'SCORPION',
             u'SCYTHE', u'SCYTHE FLEET ISSUE', u'SENTINEL', u'SIGIL',
             u'SILVER MAGNATE', u'SIN', u'SKIFF', u'SLASHER', u'SLEIPNIR',
             u'STABBER', u'STABBER FLEET ISSUE', u'STILETTO', u'STRATIOS',
             u'SUCCUBUS', u'TALOS', u'TALWAR', u'TARANIS', u'TEMPEST',
             u'TEMPEST FLEET ISSUE', u'TEMPEST TRIBAL ISSUE', u'TENGU',
             u'THANATOS', u'THORAX', u'THRASHER', u'TORMENTOR', u'TORNADO',
             u'TRISTAN', u'TYPHOON', u'VAGABOND', u'VARGUR', u'VELATOR',
             u'VENGEANCE', u'VEXOR', u'VEXOR NAVY ISSUE', u'VIATOR', u'VIGIL',
             u'VIGILANT', u'VINDICATOR', u'VULTURE', u'WIDOW', u'WOLF',
             u'WORM', u'WREATHE', u'WYVERN', u'ZEALOT', u'CAPSULE',]
SHIPNAMES = sorted(SHIPNAMES, key=lambda x: len(x), reverse=True)